/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2011, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** serial i/o module                                                 Rev 0.0  04/17/2011 **/
/**                                                                                       **/
/*******************************************************************************************/
module sio_top (s_clk_out, s_data_out, us_bufr_done, us_bufr_empt, us_bufr_full, us_bufr_ovr, 
                us_data_out, u_data_out, clkp, cks_mode, div_rate, s_data_in, s_reset,
                us_bufr_rd, us_bufr_wr, us_data_in, u_data_in);

  input         clkp;          /* main peripheral clock                                    */
  input         cks_mode;      /* sdio mode                                                */
  input         s_data_in;     /* sd rx data input                                         */
  input         s_reset;       /* synchronous reset                                        */
  input         us_bufr_rd;    /* serial rx buffer read                                    */
  input         us_bufr_wr;    /* serial tx buffer write                                   */
  input         u_data_in;     /* uart rx data input                                       */
  input   [7:0] us_data_in;    /* data bus input                                           */
  input  [11:0] div_rate;      /* serial baud rate divider                                 */

  output        s_clk_out;     /* sd clk output                                            */
  output        s_data_out;    /* sd data output                                           */
  output        us_bufr_done;  /* serial tx done sending                                   */
  output        us_bufr_empt;  /* serial tx buffer empty                                   */
  output        us_bufr_full;  /* serial rx buffer full                                    */
  output        us_bufr_ovr;   /* serial rx buffer overrun                                 */
  output        u_data_out;    /* uart data output                                         */
  output  [7:0] us_data_out;   /* data bus output                                          */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        auto_trig;                                   /* clocked serial trigger       */
  wire        rx_run;                                      /* rcvr running (cks mode)      */
  wire        rx_sync;                                     /* internal rx clock            */
  wire        s_clk_out;                                   /* sd clk output                */
  wire        s_data_out;                                  /* sd data output               */
  wire        serx_rxd;                                    /* receiver data input          */
  wire        serx_tdat;                                   /* transmitter data output      */
  wire        tx_run;                                      /* xmtr running (cks mode)      */
  wire        tx_sync;                                     /* internal tx clock            */
  wire        u_data_out;                                  /* uart data output             */
  wire        us_bufr_done;                                /* serial tx done sending       */
  wire        us_bufr_empt;                                /* serial tx buffer empty       */
  wire        us_bufr_full;                                /* serial rx buffer full        */
  wire        us_bufr_ovr;                                 /* serial rx buffer overrun     */
  wire  [7:0] us_data_out;                                 /* data bus output              */

  reg         serx_iclk;                                   /* internal clock               */
  reg         serx_divpls;                                 /* divider output pulse         */
  reg  [11:0] serx_divcnt;                                 /* divider counter              */

  /*****************************************************************************************/
  /*                                                                                       */
  /* baud rate divider                                                                     */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clkp) begin
    serx_divcnt <= (s_reset) ? 12'h3a8 :
                   (~|serx_divcnt) ? div_rate : (serx_divcnt - 1'b1);
    serx_divpls <= !s_reset && ~|serx_divcnt;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* sdio clk generator                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clkp) begin
    if (s_reset || serx_divpls) serx_iclk <= s_reset || (!rx_run && !tx_run) || !serx_iclk;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* input & output muxes                                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  assign rx_sync = (cks_mode) ? (serx_divpls && !serx_iclk) : serx_divpls;
  assign tx_sync = (cks_mode) ? (serx_divpls &&  serx_iclk) : serx_divpls;

  assign serx_rxd   = (cks_mode) ? s_data_in : u_data_in;

  assign s_clk_out  =  serx_iclk;
  assign s_data_out = !cks_mode || serx_tdat;
  assign u_data_out =  cks_mode || serx_tdat;

  /*****************************************************************************************/
  /*                                                                                       */
  /* serial port receiver and transmitter                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  srx_byte RXCV ( .bufr_ovr(us_bufr_ovr), .bufr_reg(us_data_out), .bufr0_full(us_bufr_full),
                  .rx_run(rx_run), .auto_trig(auto_trig), .cks_mode(cks_mode), .clkp(clkp),
                  .rd_sxdr(us_bufr_rd), .rx_sync(rx_sync), .s_reset(s_reset),
                  .serx_rxd(serx_rxd) );

  stx_byte XMIT ( .bufr_done(us_bufr_done), .bufr_empty(us_bufr_empt), .serx_tdat(serx_tdat),
                  .tx_run(tx_run), .auto_trig(auto_trig), .cks_mode(cks_mode), .clkp(clkp),
                  .ld_sxdr(us_bufr_wr), .pwrite_bus(us_data_in), .s_reset(s_reset),
                  .tx_sync(tx_sync) );

  endmodule






